/*
LCD4b.c
---------------------------------------------------------------------------------------
                 LCD-Routine fr KS0070 und HD44780 im 4Bit Modus v1.2
---------------------------------------------------------------------------------------
*/

#include <avr/io.h>
#include "lcd4b.h"
#include <avr/delay.h>


// sendet ein Datenbyte an das LCD
void lcd_data(char byte)
{
	char sich_byte;
	sich_byte = byte;
	#if HalfByteSel == 1
	byte = byte<<4;					//Nibbles vertauschen
	byte &= ~0xf0;					//Bit 4-7 auf 0 setzen
	#else
	byte &= ~0x0f;					//Bit 0-3 auf 0 setzen
	#endif
	byte |= (1<<lcd_rs);			//RS setzen
	PORT = byte;					//oberes Nibbel Ausgeben
	lcd_enable();

	byte = sich_byte;
	#if HalfByteSel ==  0
	byte = byte<<4;					//Nibbles vertauschen
	byte &= ~0x0f;					//Bit 0-3 auf 0 setzen
	#else
	byte &= ~0xf0;					//Bit 4-7 auf 0 setzen
	#endif
	byte |= (1<<lcd_rs);			//RS setzen
	PORT = byte;					//unteres Nibbel Ausgeben
	lcd_enable();

	kpause();
}

//sendet einen Befehl an das LCD
void lcd_command(char byte)
{                        			//wie lcd_data, nur ohne RS zu setzen
	char sich_byte;
	sich_byte = byte;
    #if HalfByteSel == 1
	byte = byte<<4;					//Nibbles vertauschen
	byte &= ~0xf0;					//Bit 4-7 auf 0 setzen
	#else
	byte &= ~0x0f;					//Bit 0-3 auf 0 setzen
	#endif
	PORT = byte;					//oberes Nibbel Ausgeben
	lcd_enable();

	byte = sich_byte;
	#if HalfByteSel ==  0
	byte = byte<<4;					//Nibbles vertauschen
	byte &= ~0x0f;					//Bit 0-3 auf 0 setzen
	#else
	byte &= ~0xf0;					//Bit 4-7 auf 0 setzen
	#endif
	PORT = byte;					//unteres Nibbel Ausgeben
	lcd_enable();

	kpause();
}

//erzeugt den Enable-Puls
void lcd_enable(void)
{
	PORT |= (1<<lcd_e);					//Enable auf High
	kpause();							//kurze pause
	PORT &= ~(1<<lcd_e);   				//Enable auf Low
}                 

//Pause nach jeder bertragung
void kpause(void)
{
	//_delay_us(48);						//48us Pause
	_delay_us(100);						//48us Pause
}


//Lngere Pause fr manche Befehle
void lpause(void) 
{
	//_delay_ms(5);						//5ms Pause
	_delay_ms(10);						//5ms Pause
}

//Initialisierung
void lcd_linit(void)					//Wenn der Display gleich nachdem er mit Spannung versorgt wurde
{										//in Betrieb genommen werden soll
	unsigned char i;
	for(i=0;i<10;i++)
	{
		//_delay_ms(10);
		_delay_ms(20);
	}
	lcd_init();
}

//Initialisierung
void lcd_init(void)
{
#if HalfByteSel == 0
     PORT = 0x20;				//Interface auf 4-Bit setzen (DB5 = 1)
#else
     PORT = 0x02;				//Interface auf 4-Bit setzen (DB5 = 1)
#endif

	lcd_enable();
	lpause();
	
	lcd_command(LCDinit);		// Funktion Set
	lcd_command(LCDOn);		    // Display On
	lcd_command(CurserInit);	// Entery Mode Set
}

//Sendet den Befehl zur Lschung des Displays
void lcd_clear(void)
{
    lcd_command(0x01);
	lpause();
}

//gibt einen Text aus
void lcd_out(char *s)
{
    while (*s)						//so lange *s != '\0' also ungleich dem "String-Endezeichen"
    {
        lcd_data(*s);				//Zeichen ausgeben
        s++;
    }
}
/*
-----------------------------------------------------------------------------------------------
LCD4b developed by Johannes K.
-----------------------------------------------------------------------------------------------
*/
